--------------------------------------------------------------------------
-- File: SCREEN.E
-- 
-- Author: Normand M. Blais
-- Date  : November 2, 1998
-- 
--------------------------------------------------------------------------
include board.e
include graphics.e

constant BDCOLOR = GRAY

constant INDEX = {
{1,2,3,4},
{5,6,7,8},
{9,10,11,12},
{13,14,15,16}
}

global constant ORIG = {194,100}    
global constant STEP = 63
global constant MAXLEN = 4*STEP

constant TOPLEFTCORNER = {
 {ORIG[1],ORIG[2]},               {ORIG[1]+STEP,ORIG[2]},
 {ORIG[1]+2*STEP,ORIG[2]},        {ORIG[1]+3*STEP,ORIG[2]},
 {ORIG[1],ORIG[2]+STEP},          {ORIG[1]+STEP,ORIG[2]+STEP},
 {ORIG[1]+2*STEP,ORIG[2]+STEP},   {ORIG[1]+3*STEP,ORIG[2]+STEP},
 {ORIG[1],ORIG[2]+2*STEP},        {ORIG[1]+STEP,ORIG[2]+2*STEP},
 {ORIG[1]+2*STEP,ORIG[2]+2*STEP}, {ORIG[1]+3*STEP,ORIG[2]+2*STEP},
 {ORIG[1],ORIG[2]+3*STEP},        {ORIG[1]+STEP,ORIG[2]+3*STEP},
 {ORIG[1]+2*STEP,ORIG[2]+3*STEP}, {ORIG[1]+3*STEP,ORIG[2]+3*STEP}
}

constant TYPETOP=1, TYPEMID=2, TYPEBOT=3

constant SHAPECOORD = {
{{4,11},{11,4},{18,11},{13,17},{13,21},{9,21},{9,17}},
{{9,0},{13,0},{13,21},{9,21}},
{{4,11},{9,5},{9,0},{13,0},{13,5},{18,11},{11,18}}
}

sequence FIGA, FIGB, FIGC,NONE_

FIGA = 3*SHAPECOORD[TYPETOP]
FIGB = 3*SHAPECOORD[TYPEMID]
FIGC = 3*SHAPECOORD[TYPEBOT]
NONE_ = 3*{{0,0},{21,0},{21,21},{0,21}}

global integer freeloc

freeloc = 16             

function transX(sequence s, integer z)
    for i=1 to length(s) do
        s[i][1] = s[i][1] + z
    end for
    return s
end function

function transY(sequence s, integer z)
    for i=1 to length(s) do
        s[i][2] = s[i][2] + z
    end for 
    return s
end function

function transXY(sequence s, sequence xy)
    for i=1 to length(s) do
        s[i][1] = s[i][1] + xy[1]
        s[i][2] = s[i][2] + xy[2]
    end for 
    return s
end function


constant COLOR = {
RED,BLUE,GREEN,YELLOW,
RED,BLUE,GREEN,YELLOW,
RED,BLUE,GREEN,YELLOW,
RED,BLUE,GREEN,BLACK
}                   

constant SHAPE = {
FIGA,FIGA,FIGA,FIGA,
FIGB,FIGB,FIGB,FIGB,
FIGB,FIGB,FIGB,FIGC,
FIGC,FIGC,FIGC,NONE_
}                   

sequence colors, shapes

procedure drawShape(sequence shape,integer color, integer location)
   polygon(BDCOLOR,1,transXY(NONE_,TOPLEFTCORNER[location]))
   polygon(color,1,transXY(shape,TOPLEFTCORNER[location]))
end procedure

procedure drawGrid()
   draw_line(BLACK,{{ORIG[1],ORIG[2]},{ORIG[1],ORIG[2]+MAXLEN}})
   draw_line(BLACK,{{ORIG[1]+STEP,ORIG[2]},
                                 {ORIG[1]+STEP,ORIG[2]+MAXLEN}})
   draw_line(BLACK,{{ORIG[1]+2*STEP,ORIG[2]},
                               {ORIG[1]+2*STEP,ORIG[2]+MAXLEN}})
   draw_line(BLACK,{{ORIG[1]+3*STEP,ORIG[2]},
                               {ORIG[1]+3*STEP,ORIG[2]+MAXLEN}})
   draw_line(BLACK,{{ORIG[1]+MAXLEN,ORIG[2]},
                               {ORIG[1]+MAXLEN,ORIG[2]+MAXLEN}})
   draw_line(BLACK,{{ORIG[1],ORIG[2]+STEP},
                                     {ORIG[1]+MAXLEN,ORIG[2]+STEP}})
   draw_line(BLACK,{{ORIG[1],ORIG[2]+2*STEP},
                                   {ORIG[1]+MAXLEN,ORIG[2]+2*STEP}})
   draw_line(BLACK,{{ORIG[1],ORIG[2]+3*STEP},
                                   {ORIG[1]+MAXLEN,ORIG[2]+3*STEP}})
   polygon(BRIGHT_GREEN,0,{{ORIG[1],ORIG[2]},{ORIG[1]+MAXLEN,ORIG[2]},
           {ORIG[1]+MAXLEN,ORIG[2]+MAXLEN},{ORIG[1],ORIG[2]+MAXLEN}})
end procedure

procedure mapboards()
   integer k     
   sequence cs,ss
   cs = colors
   ss = shapes
   k = 1    
   for i=1 to 4 do
       for j = 1 to 4 do
          cs[k] = colors[link[BOARD][i][j]]    
          ss[k] = shapes[link[BOARD][i][j]] 
          k = k + 1   
       end for
   end for 
   colors = cs
   shapes = ss
   freeloc = INDEX[link[2][1]][link[2][2]]
end procedure


global
procedure drawBoard()
   colors = COLOR
   shapes = SHAPE
   mapboards()
   polygon(BDCOLOR,1,{{ORIG[1],ORIG[2]},{ORIG[1]+MAXLEN,ORIG[2]},
                   {ORIG[1]+MAXLEN,ORIG[2]+MAXLEN},{ORIG[1],ORIG[2]+MAXLEN}})
   for i=1 to 4 do
      drawShape(shapes[i],colors[i],i)
   end for
   for i=5 to 11 do
      drawShape(shapes[i],colors[i],i)
   end for
   drawShape(shapes[12],colors[12],12)
   for i=13 to 16 do
      drawShape(shapes[i],colors[i],i)
   end for
   drawGrid()
end procedure 


global
procedure RotateLeft(integer row)
   object xc, xs
   if rotateLeft(row) then
      if row = 1 then
         row = 0
      elsif row = 4 then
         row = 12
      end if
      xs = shapes[1+row]
      shapes[1+row] = shapes[2+row]
      shapes[2+row] = shapes[3+row]
      shapes[3+row] = shapes[4+row]
      shapes[4+row] = xs
      xc = colors[1+row]
      colors[1+row] = colors[2+row]
      colors[2+row] = colors[3+row]
      colors[3+row] = colors[4+row]
      colors[4+row] = xc
      for i=1+row to 4+row do
         drawShape(shapes[i],colors[i],i)
      end for
      drawGrid()
      freeloc = INDEX[link[2][1]][link[2][2]]
   end if
end procedure

global
procedure RotateRight(integer row)
   object xc, xs
   if rotateRight(row) then
      if row = 1 then
         row = 0
      elsif row = 4 then
         row = 12
      end if
      xs = shapes[4+row]
      shapes[4+row] = shapes[3+row]
      shapes[3+row] = shapes[2+row]
      shapes[2+row] = shapes[1+row]
      shapes[1+row] = xs
      xc = colors[4+row]
      colors[4+row] = colors[3+row]
      colors[3+row] = colors[2+row]
      colors[2+row] = colors[1+row]
      colors[1+row] = xc
      for i=4+row to 1+row by -1 do
         drawShape(shapes[i],colors[i],i)
      end for
      drawGrid()
      freeloc = INDEX[link[2][1]][link[2][2]]
   end if
end procedure

global
procedure PushDown()
   if pushDown() then
      shapes[freeloc] = shapes[freeloc-4]
      colors[freeloc] = colors[freeloc-4]
      shapes[freeloc-4] = NONE_
      colors[freeloc-4] = BLACK
      drawShape(NONE_,BLACK,freeloc-4)
      drawShape(shapes[freeloc],colors[freeloc],freeloc)
      drawGrid()
      freeloc = INDEX[link[FREELOC][1]][link[FREELOC][2]]
   end if
end procedure

global
procedure PushUp()
   if pushUp() then
      shapes[freeloc] = shapes[freeloc+4]
      colors[freeloc] = colors[freeloc+4]
      shapes[freeloc+4] = NONE_
      colors[freeloc+4] = BLACK
      drawShape(NONE_,BLACK,freeloc+4)
      drawShape(shapes[freeloc],colors[freeloc],freeloc)
      drawGrid()
      freeloc = INDEX[link[FREELOC][1]][link[FREELOC][2]]
   end if
end procedure

